//::///////////////////////////////////////////////
//:: tk_listener_i0
//::
//:: Dedicated listening creature include file.
//:://////////////////////////////////////////////
/*
    Include file with functions that aid in
    using a creature that will perform listening
    tasks for a placeable, etc.
*/
//:://////////////////////////////////////////////
//:: Created By: The Krit
//:: Created On: May 21, 2007
//:://////////////////////////////////////////////


//----------------------------------------------------------
// PROTOTYPES
//----------------------------------------------------------


// Creates a dedicated listener for the caller.
// This creation is not instantaneous, so this should be called at least one
// NPC conversation node before the listener needs to listen.
void CreateListener();

// Destroys the caller's dedicated listener.
void DestroyListener();

// Pauses a conversation until a response is heard from oTarget.
// If oTarget is OBJECT_INVALID, then a response from any creature will do.
// The response is stored in the local string (on the caller) named sLocal.
// A dedicated listener must have already been created via CreateListener().
// (This function will return while the conversation is paused.)
void GetResponse(string sLocal, object oTarget = OBJECT_INVALID);


//----------------------------------------------------------
// FUNCTIONS
//----------------------------------------------------------


//----------------------------------------------------------
// UnfreezeConversation()
//
// Call this on a delay to clean up the conversation in case the player aborts
// it while it's paused. (The conversation's abort script is not called in this
// case under NWN 1.68.)
//
// oListener is the listener involved.
// nCode is the control code at the time this was delay-called.
//
void UnfreezeConversation(object oListener, int nCode)
{
    // Make sure we are still in the same listening pattern.
    if ( GetLocalInt(oListener, "ControlCode") == nCode  &&  GetIsListening(oListener) )
    {
        // Inform the player.
        SendMessageToPC(GetPCSpeaker(), GetName(OBJECT_SELF) + " has stopped waiting for your response. This is to avoid an undesirable situation.");
        // Resume the conversation so that things operate normally again.
        ActionResumeConversation();
    }
}


//----------------------------------------------------------
// CreateListener()
//
// Creates a dedicated listener for the caller.
// This creation is not instantaneous, so this should be called at least one
// NPC conversation node before the listener needs to listen.
//
void CreateListener()
{
    // Create a dedicated listener.
    object oListener = CreateObject(OBJECT_TYPE_CREATURE, "tk_listener", GetLocation(OBJECT_SELF));

    // Check the creation.
    if ( oListener == OBJECT_INVALID )
        SendMessageToPC(GetPCSpeaker(), GetName(OBJECT_SELF) + " is having trouble hearing. You may have trouble with this conversation. Sorry.");
    else
    {
        // Link the listener to ourselves.
        SetLocalObject(oListener, "EmployedBy", OBJECT_SELF);
        SetLocalObject(OBJECT_SELF, "TK_Listener", oListener);
    }
}


//----------------------------------------------------------
// DestroyListener()
//
// Destroys the caller's dedicated listener.
//
void DestroyListener()
{
    object oListener = GetLocalObject(OBJECT_SELF, "TK_Listener");

    // See if there is a paused conversation.
    if ( GetIsListening(oListener) )
        // Unpause the conversation.
        ActionResumeConversation();

    // Destroy the listener.
    DestroyObject(oListener);
    // Unlink us from the former listener.
    DeleteLocalObject(OBJECT_SELF, "TK_Listener");
}


//----------------------------------------------------------
// GetResponse()
//
// Pauses a conversation until a response is heard from oTarget.
// If oTarget is OBJECT_INVALID, then a response from any creature will do.
// The response is stored in the local string (on the caller) named sLocal.
//
// A dedicated listener must have already been created via CreateListener().
//
void GetResponse(string sLocal, object oTarget = OBJECT_INVALID)
{
    // Clear out the local variable.
    SetLocalString(OBJECT_SELF, sLocal, "");

    // Get the dedicated listener.
    object oListener = GetLocalObject(OBJECT_SELF, "TK_Listener");
    // Abort if the listener does not exist.
    if ( !GetIsObjectValid(oListener) )
        return;

    // Initialize the local variables on the listener.
    SetLocalObject(oListener, "ListenTo", oTarget);
    SetLocalString(oListener, "RecordAs", sLocal);

    // Pause the conversation.
    // (Will be unpaused by the listener's OnConversation event.)
    ClearAllActions();
    ActionPauseConversation();

    // Backup plan in case player erroneously decides that escaping from the
    // paused conversation is a good idea.
    int nCode = GetLocalInt(oListener, "ControlCode") + 1;
    SetLocalInt(oListener, "ControlCode", nCode);
    DelayCommand(60.0, UnfreezeConversation(oListener, nCode));

    // Tell the listener to start listening.
    SetListening(oListener, TRUE);
}

